extern "C" {
	void *cuda_alloc(size_t size) {
		void *ptr = NULL;
		cudaError_t res = cudaMalloc(&ptr, size);
		if (res != cudaSuccess) {
			ptr = NULL;
		}
		return ptr;
	}
	
	void *cuda_alloc_zeroed(size_t size) {
		void *ptr = cuda_alloc(size);
		if (!ptr) {
			return NULL;
		}
		cudaError_t res = cudaMemset(ptr, 0, size);
		if (res != cudaSuccess) {
			return NULL;
		}
		return ptr;
	}
	
	void cuda_dealloc(void *ptr) {
		cudaFree(ptr);
	}
	
	void copy_cpu_to_cuda(void *dst, void *src, size_t size) {
		cudaMemcpy(dst, src, size, cudaMemcpyHostToDevice);
	}
	
	void copy_cuda_to_cpu(void *dst, void *src, size_t size) {
		cudaMemcpy(dst, src, size, cudaMemcpyDeviceToHost);
	}
	
	void copy_cuda_to_cuda(void *dst, void *src, size_t size) {
		cudaMemcpy(dst, src, size, cudaMemcpyDeviceToDevice);
	}
}
