use crate::ir::*;

macro_rules! pass {
    ($p:ident) => {
        ScheduleStmt::Let {
            var: String::from("_"),
            exp: ScheduleExp::RunPass {
                pass: Pass::$p,
                args: vec![],
                on: Selector::Everything(),
            },
        }
    };
}

macro_rules! default_schedule {
    () => {
        ScheduleStmt::Block {
            body: vec![],
        }
    };
    ($($p:ident),+ $(,)?) => {
        ScheduleStmt::Block {
            body: vec![$(pass!($p)),+],
        }
    };
}

// Defualt schedule, which is used if no schedule is provided
pub fn default_schedule() -> ScheduleStmt {
    default_schedule![
        GVN,
        DCE,
        PhiElim,
        DCE,
        CRC,
        DCE,
        SLF,
        DCE,
        Inline,
        /*DeleteUncalled,*/
        InterproceduralSROA,
        SROA,
        PhiElim,
        DCE,
        CCP,
        DCE,
        GVN,
        DCE,
        WritePredication,
        PhiElim,
        DCE,
        CRC,
        DCE,
        SLF,
        DCE,
        Predication,
        DCE,
        CCP,
        DCE,
        GVN,
        DCE,
        LiftDCMath,
        DCE,
        GVN,
        DCE,
        /*Forkify,*/
        /*ForkGuardElim,*/
        DCE,
        ForkSplit,
        Unforkify,
        GVN,
        DCE,
        DCE,
        AutoOutline,
        InterproceduralSROA,
        SROA,
        InferSchedules,
        DCE,
        GCM,
        DCE,
        FloatCollections,
        GCM,
    ]
}
